"""
This function is contains the default
variables for the 'sortalgo' module
"""
in_order_default_values = ['SmallToBig', 'BigToSmall']

#class quick_sort_default_values:
#    def __init__(self): pass
#
#    move_same_value_to_default_values = ['down', 'up']