import numpy as np

BoolType = type(True)

StringType = type("This is a string")
CharType = type('s')

IntType = type(5)
FloatType = type(5.5)

ListType = type([])
DictionatyType = type({1:"first", "2":"second", 3:"third"})

ByteType = type(b"hello there")
ByteArrayType = type(bytearray(b"hello there"))

ArrayType = type(np.array([]))