print("...DONE...")


# python -m pip install --user --upgrade setuptools wheel
# python -m pip install --user --upgrade twine

# python setup.py sdist bdist_wheel

# python -m twine upload --repository testpypi dist/*  <-- upload to test Pypi
# python -m twine upload dist/*  <-- upload to Pypi




#You can use
#[Github-flavored Markdown](https://guides.github.com/features/mastering-markdown/)
#to write your content.





# // Mission Log \\
#
# pytrainer --> ML_functions // add lambda to Normal_Equation
# pytrainer --> ML_functions // complete Cost_function_Grad -> linear regression \\
# pytrainer --> ML_functions // complete Cost_function -> logistic regression \\
# pytrainer --> ML_functions // complete Cost_function_Grad -> logistic regression \\
#
# pytrainer --> ML_functions // complete gradient_descent \\
# pytrainer --> ML_functions_Training // complete forward_propagation \\ (add forward propagation to neural network)